@@ -110,14 +110,16 @@ def periodogram(X, Y, per_type='per', sampling_f = '', logscale = False, name =
     elif per_type =='welch':
         # Welch
         f, Pxx_den = signal.welch(Y,sampling_f)
-    else:
+    elif per_type == 'lombscargle':
         # Lomb-Scargle
         min_per = 2
         #max_per = 50
         
         f = np.linspace(1/max_per, 1/min_per, 10)
         Pxx_den = signal.lombscargle(X, Y, f)
-        
+    else:
+        print("Invalid option")
+        return
         
 
     # significance
@@ -445,8 +447,11 @@ def generate_independents(X, n_components = 3, period = 24, lin_comp = False):
         X_fit = np.column_stack((A1, B1, A2, B2))        
     elif n_components == 3:
         X_fit = np.column_stack((A1, B1, A2, B2, A3, B3))        
-    else:
+    elif n_components == 4:
         X_fit = np.column_stack((A1, B1, A2, B2, A3, B3, A4, B4))
+    else:
+        print("Invalid option")
+        return
 
     if lin_comp and n_components:
         X_fit = np.column_stack((X, X_fit))
@@ -461,15 +466,44 @@ def population_fit(df_pop, n_components = 2, period = 24, model_type = 'lin', li
 
     tests = df_pop.test.unique()
     k = len(tests)
+    
+    X_test = np.linspace(0, 100, 1000)
+    X_fit_eval_params = generate_independents(X_test, n_components = n_components, period = period, lin_comp = lin_comp)
+    if lin_comp:
+        X_fit_eval_params[:,1] = 0    
+    
+    min_X = 1000
+    max_X = 0
+    min_Y = 1000
+    max_Y = 0
+    min_Y_test = 1000
+    max_Y_test = 0
+    min_X_test = np.min(X_test)
+    
+    
     for test in tests:
         x,y = np.array(df_pop[df_pop.test == test].x), np.array(df_pop[df_pop.test == test].y)
+        
+        min_X = min(min_X, np.min(x))
+        max_X = max(max_X, np.max(x))
+        
+        min_Y = min(min_Y, np.min(y))
+        max_Y = max(max_Y, np.max(y))
+        
+        
         results, statistics, rhythm_params, X_test, Y_test, model = fit_me(x, y, n_components = n_components, period = period, model_type = model_type, lin_comp=lin_comp, plot = False, return_model = True, plot_phase=False)
         if type(params) == int:
             params = results.params
         else:
             params = np.vstack([params, results.params])
         if plot_on and plot_individuals:
-            plt.plot(x, results.fittedvalues,'k', label=test)
+            Y_eval_params = results.predict(X_fit_eval_params)
+            plt.plot(X_test,Y_eval_params,'k', label=test)
+            
+            min_Y_test = min(min_Y_test, np.min(Y_eval_params))
+            max_Y_test = max(max_Y_test, np.max(Y_eval_params))
+            
+            #plt.plot(x, results.fittedvalues,'k', label=test)
         if plot_on and plot_measurements:
             plt.plot(x,y,'ko', markersize=1)
     
@@ -508,25 +542,36 @@ def population_fit(df_pop, n_components = 2, period = 24, model_type = 'lin', li
     X_fit = generate_independents(x, n_components = n_components, period = period, lin_comp = lin_comp)
     Y_fit = results.predict(X_fit)
     
+    
+    Y_eval_params = results.predict(X_fit_eval_params)    
+    rhythm_params = evaluate_rhythm_params(X_test, Y_eval_params)
+        
     if plot_on:
-        plt.plot(x,Y_fit,'r', label="population fit")
+        #plt.plot(x,Y_fit,'r', label="population fit")
+        plt.plot(X_test,Y_eval_params,'r', label="population fit")
         plt.legend()
         plt.xlabel('time [h]')
         plt.ylabel('measurements')
-
+    
+        min_Y_test = min(min_Y_test, np.min(Y_eval_params))
+        max_Y_test = max(max_Y_test, np.max(Y_eval_params))
+    
         
 
     if plot_on and plot_margins:
-        sdev, lower, upper = wls_prediction_std(results, exog=X_fit, alpha=0.05)
-        plt.fill_between(x, lower, upper, color='#888888', alpha=0.1)                   
+        sdev, lower, upper = wls_prediction_std(results, exog=X_fit_eval_params, alpha=0.05)
+        plt.fill_between(X_test, lower, upper, color='#888888', alpha=0.1)                   
+    
+    if plot_measurements:
+        if model_type == 'lin':
+            plt.axis([min(min_X,0), 1.1*max(max_X,period), 0.9*min(min_Y, min_Y_test), 1.1*max(max_Y, max_Y_test)])
+        else:
+            plt.axis([min(min_X,0), max_X, 0.9*min(min_Y, min_Y_test), 1.1*max(max_Y, max_Y_test)])
+        
+    else:
+        plt.axis([min_X_test, 50, min_Y_test*0.9, max_Y_test*1.1])
     
-
     
-    statistics = calculate_statistics(x, y, Y_fit, n_components, period, lin_comp) 
-    statistics_params = {'values': means,
-                        'SE': se,
-                        'CI': (lower_CI, upper_CI),
-                        'p-values': p_values} 
     if plot_on:
         pop_name = "_".join(test.split("_")[:-1])
         plt.title(pop_name + ', p-value=' + "{0:.5f}".format(statistics['p']))
@@ -539,20 +584,20 @@ def population_fit(df_pop, n_components = 2, period = 24, model_type = 'lin', li
             plt.show()
     
 
+   
+
+    statistics = calculate_statistics(x, y, Y_fit, n_components, period, lin_comp) 
+    statistics_params = {'values': means,
+                        'SE': se,
+                        'CI': (lower_CI, upper_CI),
+                        'p-values': p_values} 
 
 
-
-
-
-    X_test = np.linspace(0, 100, 1000)
-    X_fit_eval_params = generate_independents(X_test, n_components = n_components, period = period, lin_comp = lin_comp)
-    if lin_comp:
-        X_fit_eval_params[:,1] = 0    
-    Y_eval_params = results.predict(X_fit_eval_params)    
-    rhythm_params = evaluate_rhythm_params(X_test, Y_eval_params)
+    
     
     return params, statistics, statistics_params, rhythm_params, results
 
+
 def fit_me(X, Y, n_components = 2, period = 24, model_type = 'lin', lin_comp = False, alpha = 0, name = '', save_to = '', plot=True, plot_residuals=False, plot_measurements=True, plot_margins=True, return_model = False, color = False, plot_phase = True):
     """
     ###
@@ -593,9 +638,13 @@ def fit_me(X, Y, n_components = 2, period = 24, model_type = 'lin', lin_comp = F
     elif n_components == 3:
         X_fit = np.column_stack((A1, B1, A2, B2, A3, B3))
         X_fit_test = np.column_stack((A1_test, B1_test, A2_test, B2_test, A3_test, B3_test))
-    else:
+    elif n_components == 4:
         X_fit = np.column_stack((A1, B1, A2, B2, A3, B3, A4, B4))
         X_fit_test = np.column_stack((A1_test, B1_test, A2_test, B2_test, A3_test, B3_test, A4_test, B4_test))
+    else:
+        print("Invalid option")
+        return
+
     
     X_fit_eval_params = X_fit_test
     
@@ -631,7 +680,7 @@ def fit_me(X, Y, n_components = 2, period = 24, model_type = 'lin', lin_comp = F
         model = statsmodels.discrete.count_model.ZeroInflatedNegativeBinomialP(Y,X_fit,p=2)
         #results = model.fit()
         results = model.fit(method='bfgs', maxiter=5000, maxfun=5000)
-    else:
+    elif model_type == 'nb':
         #exposure = np.zeros(len(Y))
         #exposure[:] = np.mean(Y)
         #model = sm.GLM(Y, X_fit, family=sm.families.NegativeBinomial(), exposure = exposure)
@@ -640,6 +689,10 @@ def fit_me(X, Y, n_components = 2, period = 24, model_type = 'lin', lin_comp = F
         else:
             model = sm.GLM(Y, X_fit, family=sm.families.NegativeBinomial())
         results = model.fit()
+    else:
+        print("Invalid option")
+        return
+
     
     if model_type =='gen_poisson':
         Y_fit = results.predict(X_fit)
@@ -947,7 +1000,7 @@ def calculate_statistics(X, Y, Y_fit, n_components, period, lin_comp = False):
     N = Y.size
 
     F = (MSS/(n_params - 1)) / (RSS/(N - n_params)) 
-    p = 1 - stats.f.cdf(F, n_params - 1, N - n_params);
+    p = 1 - stats.f.cdf(F, n_params - 1, N - n_params)
     #print("p-value(Cornelissen): {}".format(p))
     
     # statistics of GOF according to Cornelissen (eqs (14) - (15))
@@ -967,8 +1020,12 @@ def calculate_statistics(X, Y, Y_fit, n_components, period, lin_comp = False):
     #print('SSPE: ', SSPE)
     #print('SSLOF: ', SSLOF)
     if lin_comp:
-        F = (SSLOF/(n_T-1-(2*n_components + 1)))/(SSPE/(N-n_T))
-        p_reject = 1 - stats.f.cdf(F, n_T-1-(2*n_components + 1), N-n_T)
+        try:
+            F = (SSLOF/(n_T-1-(2*n_components + 1)))/(SSPE/(N-n_T))
+            p_reject = 1 - stats.f.cdf(F, n_T-1-(2*n_components + 1), N-n_T)
+        except:
+            F = np.nan
+            p_reject = np.nan    
     else:    
         try:
             F = (SSLOF/(n_T-1-2*n_components))/(SSPE/(N-n_T))
@@ -1006,7 +1063,7 @@ def calculate_statistics_nonlinear(X, Y, Y_fit, n_params, period):
     N = Y.size
 
     F = (MSS/(n_params - 1)) / (RSS/(N - n_params)) 
-    p = 1 - stats.f.cdf(F, n_params - 1, N - n_params);
+    p = 1 - stats.f.cdf(F, n_params - 1, N - n_params)
     
     X_periodic = np.round_(X % period,2)                                    
     
@@ -1171,7 +1228,7 @@ def compare_pair_df(df, test1, test2, n_components = 3, period = 24, lin_comp =
             X_fit = np.column_stack((A1, B1, A2, B2, A3, B3, A1_i, B1_i, A2_i, B2_i, A3_i, B3_i, H_i))
             #idx_params = [7, 8, 9, 10, 11, 12, 13]
             idx_params = [7, 8, 9, 10, 11, 12]
-    else:
+    elif n_components == 4:
         if non_rhythmic:
             X_fit = np.column_stack((A1, B1, A2, B2, A3, B3, A4, B4, H_i))
             idx_params = [9]
@@ -1179,6 +1236,10 @@ def compare_pair_df(df, test1, test2, n_components = 3, period = 24, lin_comp =
             X_fit = np.column_stack((A1, B1, A2, B2, A3, B3, A4, B4, A1_i, B1_i, A2_i, B2_i, A3_i, B3_i, A4_i, B4_i, H_i))
             #idx_params = [9, 10, 11, 12, 13, 14, 15, 16, 17]
             idx_params = [9, 10, 11, 12, 13, 14, 15, 16]
+    else:
+        print("Invalid option")
+        return
+
            
         
     if lin_comp:
@@ -1210,7 +1271,7 @@ def compare_pair_df(df, test1, test2, n_components = 3, period = 24, lin_comp =
         model = statsmodels.discrete.count_model.ZeroInflatedNegativeBinomialP(Y,X,p=2)
         #results = model.fit()
         results = model.fit(method='bfgs', maxiter=5000, maxfun=5000)
-    else:
+    elif model_type == 'nb':
         #exposure = np.zeros(len(Y))
         #exposure[:] = np.mean(Y)
         #model = sm.GLM(Y, X, family=sm.families.NegativeBinomial(), exposure = exposure)
@@ -1219,6 +1280,10 @@ def compare_pair_df(df, test1, test2, n_components = 3, period = 24, lin_comp =
         else:
             model = sm.GLM(Y, X, family=sm.families.NegativeBinomial())
         results = model.fit()
+    else:
+        print("Invalid option")
+        return
+
     
     
     """
@@ -1366,7 +1431,7 @@ def compare_pair(X1, Y1, X2, Y2, test1 = '', test2 = '', n_components = 3, perio
             X_fit = np.column_stack((A1, B1, A2, B2, A3, B3, A1_i, B1_i, A2_i, B2_i, A3_i, B3_i, H_i))
             #idx_params = [7, 8, 9, 10, 11, 12, 13]
             idx_params = [7, 8, 9, 10, 11, 12]
-    else:
+    elif n_components == 4:
         if non_rhythmic:
             X_fit = np.column_stack((A1, B1, A2, B2, A3, B3, A4, B4, H_i))
             idx_params = [9]
@@ -1374,6 +1439,10 @@ def compare_pair(X1, Y1, X2, Y2, test1 = '', test2 = '', n_components = 3, perio
             X_fit = np.column_stack((A1, B1, A2, B2, A3, B3, A4, B4, A1_i, B1_i, A2_i, B2_i, A3_i, B3_i, A4_i, B4_i, H_i))
             #idx_params = [9, 10, 11, 12, 13, 14, 15, 16, 17]
             idx_params = [9, 10, 11, 12, 13, 14, 15, 16]
+    else:
+        print("Invalid option")
+        return
+
            
     if lin_comp:
         X_fit = np.column_stack((X_i, X_fit))
@@ -1404,7 +1473,7 @@ def compare_pair(X1, Y1, X2, Y2, test1 = '', test2 = '', n_components = 3, perio
         model = statsmodels.discrete.count_model.ZeroInflatedNegativeBinomialP(Y,X_fit,p=2)
         #results = model.fit()
         results = model.fit(method='bfgs', maxiter=5000, maxfun=5000)
-    else:
+    elif model_type == 'nb':
         #exposure = np.zeros(len(Y))
         #exposure[:] = np.mean(Y)
         #model = sm.GLM(Y, X, family=sm.families.NegativeBinomial(), exposure = exposure)
@@ -1413,7 +1482,10 @@ def compare_pair(X1, Y1, X2, Y2, test1 = '', test2 = '', n_components = 3, perio
         else:
             model = sm.GLM(Y, X_fit, family=sm.families.NegativeBinomial())
         results = model.fit()
-    
+    else:
+        print("Invalid option")
+        return
+
     
     """
     ###
@@ -1524,11 +1596,15 @@ def generate_independents_compare(X1, X2, n_components1 = 3, period1 = 24, n_com
             X_fit = np.column_stack((A1, B1, A2, B2, A3, B3, H_i))            
         else:
             X_fit = np.column_stack((A1, B1, A2, B2, A3, B3))            
-    else:
+    elif n_components1 == 4:
         if non_rhythmic:
             X_fit = np.column_stack((A1, B1, A2, B2, A3, B3, A4, B4, H_i))            
         else:
             X_fit = np.column_stack((A1, B1, A2, B2, A3, B3, A4, B4))            
+    else:
+        print("Invalid option")
+        return
+
         
     if n_components2 == 1:
         if not non_rhythmic:
@@ -1539,9 +1615,13 @@ def generate_independents_compare(X1, X2, n_components1 = 3, period1 = 24, n_com
     elif n_components2 == 3:
         if not non_rhythmic:
             X_fit = np.column_stack((X_fit, np.column_stack((A1_i, B1_i, A2_i, B2_i, A3_i, B3_i, H_i))))
-    else:
+    elif n_components2 == 4:
         if not non_rhythmic:
             X_fit = np.column_stack((X_fit, np.column_stack((A1_i, B1_i, A2_i, B2_i, A3_i, B3_i, A4_i, B4_i, H_i))))
+    else:
+        print("Invalid option")
+        return
+
     if lin_comp:
         X_fit = np.column_stack((X_i, X_fit))
         X_fit = np.column_stack((X, X_fit))    
@@ -1599,12 +1679,16 @@ def compare_pair_extended(X1, Y1, X2, Y2, test1 = '', test2 = '', n_components1
             idx_params = [-1]
         else:
             X_fit = np.column_stack((A1, B1, A2, B2, A3, B3))            
-    else:
+    elif n_components1 == 4:
         if non_rhythmic:
             X_fit = np.column_stack((A1, B1, A2, B2, A3, B3, A4, B4, H_i))
             idx_params = [-1]
         else:
             X_fit = np.column_stack((A1, B1, A2, B2, A3, B3, A4, B4))            
+    else:
+        print("Invalid option")
+        return
+
         
     if n_components2 == 1:
         if not non_rhythmic:
@@ -1621,11 +1705,15 @@ def compare_pair_extended(X1, Y1, X2, Y2, test1 = '', test2 = '', n_components1
             X_fit = np.column_stack((X_fit, np.column_stack((A1_i, B1_i, A2_i, B2_i, A3_i, B3_i, H_i))))                                    
             #idx_params = [-7, -6, -5, -4, -3, -2, -1]
             idx_params = [-7, -6, -5, -4, -3, -2]
-    else:
+    elif n_components2 == 4:
         if not non_rhythmic:
             X_fit = np.column_stack((X_fit, np.column_stack((A1_i, B1_i, A2_i, B2_i, A3_i, B3_i, A4_i, B4_i, H_i))))
             #idx_params = [-9, -8, -7, -6, -5, -4, -3, -2, -1]
             idx_params = [-9, -8, -7, -6, -5, -4, -3, -2]
+    else:
+        print("Invalid option")
+        return
+
     if lin_comp:
         X_fit = np.column_stack((X_i, X_fit))
         X_fit = np.column_stack((X, X_fit))    
@@ -1653,7 +1741,7 @@ def compare_pair_extended(X1, Y1, X2, Y2, test1 = '', test2 = '', n_components1
         model = statsmodels.discrete.count_model.ZeroInflatedNegativeBinomialP(Y,X_fit,p=2)
         #results = model.fit()
         results = model.fit(method='bfgs', maxiter=5000, maxfun=5000)
-    else:
+    elif model_type == 'nb':
         #exposure = np.zeros(len(Y))
         #exposure[:] = np.mean(Y)
         #model = sm.GLM(Y, X, family=sm.families.NegativeBinomial(), exposure = exposure)
@@ -1662,6 +1750,10 @@ def compare_pair_extended(X1, Y1, X2, Y2, test1 = '', test2 = '', n_components1
         else:
             model = sm.GLM(Y, X_fit, family=sm.families.NegativeBinomial())
         results = model.fit()
+    else:
+        print("Invalid option")
+        return
+
     
     
     """
@@ -1834,11 +1926,7 @@ def ct_test(count, poiss_results):
     return alpha       
 
 
-"""
-primerjava med rezimi:
-- ANOVA. Razmerje med variancami povprecij skupin in varianco na celotnem vzorcu.
-- Verjetno v redu, ce menjava rezima vpliva na amplitudo, ne pa, ce menjava vpliva na fazo ali periodo
-"""
+
 def compare_ANOVA(df, pairs, n_components = 3, period = 24):
     # https://pythonfordatascience.org/anova-python/
     # http://www.statistik.si/storitve/statisticne-analize-testi/anova-analiza-variance/
@@ -1887,8 +1975,6 @@ def test_phase(X1, Y1, X2, Y2, phase, period = 0, test1 = '', test2 = ''):
         X1 %= period
         X2 %= period
     
-    # test ali so variance enake - nasprotno od ANOVA
-    # t-test v vsaki tocki. V vseh tockah moras zavrniti hipotezo, da sta porazdelitvi razlicni
 
 
 def amp_phase_fit(predictor, Amp, Per, Phase, Base, dAmp, dPhase, dBase):
